# =====================================
# generator=datazen
# version=2.1.0
# hash=c899afd4b2308889f0dc1260eede4d46
# =====================================
"""
svgen - Package's default entry-point.
"""

# built-in
import sys

# internal
from svgen.entry import main

if __name__ == "__main__":
    sys.exit(main(sys.argv))
