# =====================================
# generator=datazen
# version=2.1.0
# hash=21abdf006353a5984d0a37e7e2854553
# =====================================

"""
Useful defaults and other package metadata.
"""

DESCRIPTION = "A simple interface to setuptools's setup function."
PKG_NAME = "setuptools-wrapper"
VERSION = "0.1.2"
