# =====================================
# generator=datazen
# version=2.1.0
# hash=6bb3085a425c3a2b745ee35ac0fe5b3e
# =====================================

"""
Useful defaults and other package metadata.
"""

DESCRIPTION = "A tool for working with scalable vector graphics."
PKG_NAME = "svgen"
VERSION = "0.4.0"
